<?php
/*
Plugin Name: SolWall Payment Gateway
Plugin URI: https://solwall.org/wp-plugin
Description: A Solana Wallet Payment Gateway for WooCommerce.
Version: 1.5
Author: Solwall
Author URI: https://solwall.org
License: GPLv3
License URI: http://www.gnu.org/licenses/gpl-3.0.html
*/

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

// Ensure WooCommerce is active
add_action( 'plugins_loaded', 'solwall_gateway_init', 11 );

function solwall_gateway_init() {
    if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
        return; // WooCommerce not active.
    }

    if ( class_exists( 'WC_Gateway_Solwall' ) ) {
        return; // Solwall Gateway already defined.
    }

    class WC_Gateway_Solwall extends WC_Payment_Gateway {

        /**
         * Constructor for the gateway.
         */
        public function __construct() {
            $this->id                 = 'solwall';
            $this->method_title       = __( 'Solwall Gateway', 'solwall-payment-gateway' );
            $this->method_description = __( 'Custom payment gateway for Solwall API.', 'solwall-payment-gateway' );
            $this->has_fields         = false; // Set to true if additional fields are needed on checkout.
            $this->icon = plugins_url( 'assets/icon-mini.png', __FILE__ );

            // Load the settings.
            $this->init_form_fields();
            $this->init_settings();

            // Define user set variables.
            $this->enabled        = $this->get_option( 'enabled' );
            $this->title          = $this->get_option( 'title' );
            $this->description    = $this->get_option( 'description' );
            $this->api_key        = $this->get_option( 'api_key' );
            $this->enable_logging = 'yes' === $this->get_option( 'enable_logging' ) ? true : false;

            // Define supports
            $this->supports = array(
                'products',
                // 'refunds', // Uncomment if you plan to support refunds.
            );

            // Logging
            if ( $this->enable_logging ) {
                $this->log = new WC_Logger();
                $this->log->add( 'solwall', 'Solwall Gateway Initialized' );
            }

            // Actions
            add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
            add_action( 'woocommerce_thankyou_' . $this->id, array( $this, 'thankyou_page' ) );

            // Optional: Add any additional actions or hooks here.
        }

        /**
         * Initialize Gateway Settings Form Fields
         */
        public function init_form_fields() {
            $this->form_fields = array(
                'enabled' => array(
                    'title'       => __( 'Enable/Disable', 'solwall-payment-gateway' ),
                    'type'        => 'checkbox',
                    'label'       => __( 'Enable Solwall Payment', 'solwall-payment-gateway' ),
                    'default'     => 'yes',
                ),
                'title' => array(
                    'title'       => __( 'Title', 'solwall-payment-gateway' ),
                    'type'        => 'text',
                    'description' => __( 'This controls the title which the user sees during checkout.', 'solwall-payment-gateway' ),
                    'default'     => __( 'Solwall Payment', 'solwall-payment-gateway' ),
                    'desc_tip'    => true,
                ),
                'description' => array(
                    'title'       => __( 'Description', 'solwall-payment-gateway' ),
                    'type'        => 'textarea',
                    'description' => __( 'Payment method description that the customer will see on your checkout.', 'solwall-payment-gateway' ),
                    'default'     => __( 'Pay securely using Solwall.', 'solwall-payment-gateway' ),
                ),
                'api_key' => array(
                    'title'       => __( 'API Key', 'solwall-payment-gateway' ),
                    'type'        => 'text',
                    'description' => __( 'Enter your Solwall API key.', 'solwall-payment-gateway' ),
                    'default'     => '',
                    'desc_tip'    => true,
                ),
                'enable_logging' => array(
                    'title'       => __( 'Enable Logging', 'solwall-payment-gateway' ),
                    'type'        => 'checkbox',
                    'label'       => __( 'Enable Solwall Gateway Logging', 'solwall-payment-gateway' ),
                    'default'     => 'no',
                    'description' => __( 'Log Solwall events, such as API requests, inside WooCommerce.', 'solwall-payment-gateway' ),
                ),
            );
        }

        /**
         * Check if the gateway is available for use
         *
         * @return bool
         */
        public function is_available() {
            if ( 'yes' !== $this->enabled ) {
                return false;
            }

            // Example: Check if currency is supported by Solwall.
            // Replace with actual supported currencies.
            $supported_currencies = array( 'USD', 'EUR', 'GBP' ); // Update as per Solwall's supported currencies.

            if ( ! in_array( get_woocommerce_currency(), $supported_currencies ) ) {
                if ( $this->enable_logging ) {
                    $this->log->add( 'solwall', 'Currency ' . get_woocommerce_currency() . ' not supported by Solwall.' );
                }
                return false;
            }

            // Add additional conditions if necessary (e.g., customer location, minimum order total).

            return true;
        }

        /**
         * Display description on the thank you page and open embed in new tab
         *
         * @param int $order_id
         */
        public function thankyou_page( $order_id ) {
            if ( $this->description ) {
                wpautop( wptexturize( $this->description ) );
            }

            $embed_url = get_post_meta( $order_id, '_solwall_embed_url', true );
            
             // Log order details
            if ( $this->enable_logging ) {
                $this->log->add( 'solwall', 'EMBED URL: ' . $embed_url );
            }

            if ( $embed_url != "" ) {
                ?>
                <script type="text/javascript">
                    document.addEventListener('DOMContentLoaded', function() {
                        window.open('<?php echo esc_url( $embed_url ); ?>', '_blank');
                    });
                </script>
                <p><?php esc_html_e( 'Your payment is being processed. If the payment window did not open automatically, please click the button below.', 'solwall-payment-gateway' ); ?></p>
                <a href="<?php echo esc_url( $embed_url ); ?>" target="_blank"  style="border-radius: 8px; background: black; color: white; text-decoration: none; padding: 10px;" class="button alt">
                   <img src="<?php echo esc_url( plugins_url( 'assets/icon.png', __FILE__ ) ); ?>" style="width: 32px; transform: translateY(10px);"> <?php esc_html_e( 'Complete Payment', 'solwall-payment-gateway' ); ?>
                </a>
                <?php
            }
        }

        /**
         * Process the payment and return the result
         *
         * @param int $order_id
         * @return array
         */
        public function process_payment( $order_id ) {
            $order = wc_get_order( $order_id );

            // Send API request to Solwall
            $response = $this->send_payment_request( $order );

            if ( $response['success'] ) {
                // Reduce stock levels
                wc_reduce_stock_levels( $order_id );

                // Mark as on-hold (we're awaiting the payment)
                $order->update_status( 'on-hold', __( 'Awaiting Solwall payment', 'solwall-payment-gateway' ) );

                // Remove cart
                WC()->cart->empty_cart();

                // Store the embed URL in order meta
                if ( isset( $response['embed_url'] ) ) {
                    update_post_meta( $order_id, '_solwall_embed_url', $response['embed_url'] );
                }

                // Start payment status check
                if ( isset( $response['payment_id'] ) ) {
                    $this->schedule_payment_status_check( $order_id, $response['payment_id'] );
                }

                // Return thank you page redirect
                return array(
                    'result'   => 'success',
                    'redirect' => $this->get_return_url( $order ),
                );
            } else {
                wc_add_notice( __( 'Payment error:', 'solwall-payment-gateway' ) . $response['message'], 'error' );
                return;
            }
        }

        /**
         * Send payment request to Solwall API
         *
         * @param WC_Order $order
         * @return array
         */
        private function send_payment_request( $order ) {
            $amount   = $order->get_total();
            $api_key  = $this->api_key;
            $currency = get_woocommerce_currency();

            $body = array(
                'amount'    => $amount,
                'order_id'  => $order->get_id(),
                'currency'  => $currency,
                // Add other necessary parameters as per Solwall API documentation
            );
            
            $amount = round($amount);
            // Corrected request URL, removed "GET " from the URL
            $requestUrl = "https://solwall.org/public-api/v1/createEmbed?amount=" . urlencode($amount) . "&api_key=" . urlencode($api_key);
            
             // Log order details
            if ( $this->enable_logging ) {
                $this->log->add( 'solwall', 'Order ID: ' . $order->get_id() );
                $this->log->add( 'solwall', 'Order Status: ' . $order->get_status() );
                $this->log->add( 'solwall', 'Order Total: ' . $order->get_total() );
                $this->log->add( 'solwall', 'Order Currency: ' . $order->get_currency() );
                $this->log->add( 'solwall', 'URL: ' . $requestUrl );
            }

          

            $response = wp_remote_get( $requestUrl );

            if ( $this->enable_logging ) {
                if ( is_wp_error( $response ) ) {
                    $this->log->add( 'solwall', 'Payment Request Error: ' . $response->get_error_message() );
                } else {
                    $this->log->add( 'solwall', 'Payment Request Response: ' . wp_remote_retrieve_body( $response ) );
                }
            }

            if ( is_wp_error( $response ) ) {
                return array(
                    'success' => false,
                    'message' => $response->get_error_message(),
                );
            } else {
                $body = json_decode( wp_remote_retrieve_body( $response ), true );
                if ( isset( $body['invoice_id'] ) && isset( $body['embed_url'] ) && isset( $body['callback_secret'] ) ) {
                    return array(
                        'success'          => true,
                        'payment_id'       => $body['invoice_id'],
                        'embed_url'        => $body['embed_url'],
                        'callback_secret'  => $body['callback_secret'],
                    );
                } else {
                    return array(
                        'success' => false,
                        'message' => isset( $body['error'] ) ? $body['error'] : 'Invalid response from payment gateway.',
                    );
                }
            }
        }

        /**
         * Schedule payment status checks
         *
         * @param int    $order_id
         * @param string $payment_id
         */
        private function schedule_payment_status_check( $order_id, $payment_id ) {
            update_post_meta( $order_id, '_solwall_payment_id', $payment_id );
            wp_schedule_single_event( time() + 60, 'solwall_check_payment_status', array( $order_id, 1 ) );
        }

    }

    /**
     * Add the Solwall Gateway to WooCommerce
     *
     * @param array $methods
     * @return array
     */
    function add_solwall_gateway( $methods ) {
        if ( class_exists( 'WC_Gateway_Solwall' ) ) {
            $methods[] = 'WC_Gateway_Solwall';
            // Optional: Log that the gateway was added.
            $logger = wc_get_logger();
            $logger->info( 'Solwall Gateway added to WooCommerce payment gateways.', array( 'source' => 'solwall' ) );
        } else {
            // Optional: Log that the gateway class does not exist.
            $logger = wc_get_logger();
            $logger->error( 'WC_Gateway_Solwall class does not exist.', array( 'source' => 'solwall' ) );
        }
        return $methods;
    }

    add_filter( 'woocommerce_payment_gateways', 'add_solwall_gateway' );

    /**
     * Handle payment status checks via scheduled events
     *
     * @param int $order_id
     * @param int $attempt
     */
    function solwall_check_payment_status( $order_id, $attempt ) {
        $order      = wc_get_order( $order_id );
        $payment_id = get_post_meta( $order_id, '_solwall_payment_id', true );

        if ( ! $payment_id ) {
            if ( function_exists( 'wc_get_logger' ) ) {
                $logger = wc_get_logger();
                $logger->error( 'Solwall Payment ID not found for order ' . $order_id, array( 'source' => 'solwall' ) );
            }
            return;
        }

        // Retrieve the gateway instance
        $available_gateways = WC()->payment_gateways->get_available_payment_gateways();
        if ( isset( $available_gateways['solwall'] ) ) {
            $gateway = $available_gateways['solwall'];
        } else {
            // Gateway not available
            if ( function_exists( 'wc_get_logger' ) ) {
                $logger = wc_get_logger();
                $logger->error( 'Solwall Gateway not available for payment status check.', array( 'source' => 'solwall' ) );
            }
            return;
        }

        $api_key = $gateway->api_key;

        // Corrected requestUrl and using it in wp_remote_get
        $requestUrl = "https://solwall.org/public-api/v1/checkPaymentStatus?invoice=" . urlencode($payment_id) . "&api_key=" . urlencode($api_key);
        $response = wp_remote_get( $requestUrl );

        if ( $gateway->enable_logging ) {
            if ( is_wp_error( $response ) ) {
                $gateway->log->add( 'solwall', 'Payment Status Check Error for Order ' . $order_id . ': ' . $response->get_error_message() );
            } else {
                $gateway->log->add( 'solwall', 'Payment Status Check Response for Order ' . $order_id . ': ' . wp_remote_retrieve_body( $response ) );
            }
        }

        if ( is_wp_error( $response ) ) {
            // Handle error (e.g., retry or notify admin)
            $order->add_order_note( __( 'Error checking payment status with Solwall.', 'solwall-payment-gateway' ) );
        } else {
            $body = json_decode( wp_remote_retrieve_body( $response ), true );
            if ( isset( $body['status'] ) && $body['status'] === 'completed' ) {
                $order->payment_complete();
                $order->add_order_note( __( 'Payment received via Solwall.', 'solwall-payment-gateway' ) );
                // Clear scheduled event
                wp_clear_scheduled_hook( 'solwall_check_payment_status', array( $order_id, $attempt ) );
            } else {
                if ( $attempt < 30 ) { // Max 30 attempts (30 minutes)
                    wp_schedule_single_event( time() + 60, 'solwall_check_payment_status', array( $order_id, $attempt + 1 ) );
                } else {
                    $order->update_status( 'failed', __( 'Payment not completed after 30 minutes.', 'solwall-payment-gateway' ) );
                }
            }
        }
    }

    add_action( 'solwall_check_payment_status', 'solwall_check_payment_status', 10, 2 );
}
?>
